#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <atlimage.h>

BOOL BitmapToStream(HBITMAP hbm, IStream **ppStream)
{
	if (hbm == NULL)
		return FALSE;

	CImage   img;
	IStream* ps = NULL;
	HRESULT  hr = CreateStreamOnHGlobal(NULL, TRUE, &ps);

	if (FAILED(hr))
		return FALSE;

	img.Attach(hbm);
	hr = img.Save(ps, Gdiplus::ImageFormatBMP);
	if (FAILED(hr))
	{
		ps->Release();
		return FALSE;
	}

	*ppStream = ps;
	return TRUE;
}

void DumpStream(IStream* ps)
{
	ULONG   read;
	HRESULT hr;
	BYTE    line[16];
	int     len = 16;

	/* rewind */
	LARGE_INTEGER  beg = {0};
	ULARGE_INTEGER pos = {0};
	ps->Seek(beg, STREAM_SEEK_SET, NULL);

	do {
		read = 0;
		/* get current position */
		ps->Seek(beg, STREAM_SEEK_CUR, &pos);
		/* read next chunk */
		hr = ps->Read(line, len, &read);
		if (SUCCEEDED(hr))
		{
			if (read != 0)
				printf("%08X: ", pos.LowPart);

			for (int idx = 0; idx < read; idx++)
			{
				printf(" %02X", line[idx]);
			}
			printf("\n");
		}
	} while (SUCCEEDED(hr) && read != 0);
}

int _tmain(int atrgc, TCHAR** argv)
{
	IStream* ps = NULL;
	HBITMAP hbm = (HBITMAP)::LoadImage(0, argv[1], IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION|LR_LOADFROMFILE);

	if (hbm != NULL && BitmapToStream(hbm, &ps))
	{
		DumpStream(ps);
	}

	if (ps != NULL)
		ps->Release();

	return 0;
}